/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            messagebox.h
 *
 *  Fri Feb 25 20:23:19 CET 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "config.h"
#ifdef USE_GUI

#ifndef __MIAV_MESSAGEBOX_H__
#define __MIAV_MESSAGEBOX_H__

#include <qdialog.h>
#include <qwidget.h>
#include <qframe.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qpixmap.h>

typedef enum {
  ICON_NONE,    // No icon is used
  ICON_DEFAULT, // An icon matching the buttons is used
  ICON_INFO,    // An info icon (matching the ok button)
  ICON_WARNING, // An warning icon (matching the ok/cancel button)
  ICON_ERROR,   // An critical error icon
  ICON_QUESTION // An question icon (matching the yes/no and yes/no/cancel buttons)
} msg_icon;

typedef enum {
  MSG_YES,
  MSG_NO,
  MSG_CANCEL,
  MSG_OK,
  MSG_MAYBE
} msg_val;

typedef enum {
  TYPE_OK,
  TYPE_OK_CANCEL,
  TYPE_YES_NO,
  TYPE_YES_NO_MAYBE,
  TYPE_YES_NO_CANCEL,
  TYPE_YES_NO_MAYBE_CANCEL,
} msg_type;


/** 
 * Textstrings
 */
#define TXT_OK     "Ok"
#define TXT_CANCEL "Annullr"
#define TXT_YES    "Ja"
#define TXT_NO     "Nej"
#define TXT_MAYBE  "Mske"

/** 
 * Images
 */
#define PIXMAP_INFO     PIXMAPS"/info.png"
#define PIXMAP_WARNING  PIXMAPS"/warning.png"
#define PIXMAP_QUESTION PIXMAPS"/question.png"
#define PIXMAP_ERROR    PIXMAPS"/error.png"


class MessageBox : public QDialog
{
	Q_OBJECT
public:
	MessageBox(QWidget* parent = 0, 
             const char* name = "", 
             const char* text = "", 
             msg_type type = TYPE_OK, 
             msg_icon icon = ICON_DEFAULT);
	~MessageBox();

public slots:
  void bok_clicked();
  void bcancel_clicked();
  void byes_clicked();
  void bno_clicked();
  void bmaybe_clicked();

private:
  QPixmap *pix_icon;
  QPushButton *createButton(QWidget *parent, const char *text);
};

#endif/*__MIAV_MESSAGEBOX_H__*/

#endif /*USE_GUI*/
